@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2004  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 2000-2003  Internet Software Consortium.
+ * Copyright (C) 2004, 2007  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2000-2002  Internet Software Consortium.
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -15,15 +15,15 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: lwtest.c,v 1.22.2.5 2004/03/09 06:09:59 marka Exp $ */
+/* $Id: lwtest.c,v 1.26.18.2 2007/09/13 23:46:26 tbox Exp $ */
 
 #include <config.h>
 
 #include <assert.h>
 #include <stdlib.h>
-#include <string.h>
 
 #include <isc/net.h>
+#include <isc/string.h>
 
 #include <lwres/lwres.h>
 #include <lwres/netdb.h>
@@ -695,11 +695,11 @@ main(void) {
 	test_gnba("10.10.10.17", LWRES_ADDRTYPE_V4, LWRES_R_NOTFOUND,
 		  NULL);
 	test_gnba("0123:4567:89ab:cdef:0123:4567:89ab:cdef",
-		  LWRES_ADDRTYPE_V6, LWRES_R_SUCCESS, "nibble.example");
+		  LWRES_ADDRTYPE_V6, LWRES_R_SUCCESS, "ip6.int.example");
 	test_gnba("0123:4567:89ab:cdef:0123:4567:89ab:cde0",
 		  LWRES_ADDRTYPE_V6, LWRES_R_NOTFOUND, NULL);
 	test_gnba("1123:4567:89ab:cdef:0123:4567:89ab:cdef",
-		  LWRES_ADDRTYPE_V6, LWRES_R_SUCCESS, "bitstring.example");
+		  LWRES_ADDRTYPE_V6, LWRES_R_SUCCESS, "ip6.arpa.example");
 	test_gnba("1123:4567:89ab:cdef:0123:4567:89ab:cde0",
 		  LWRES_ADDRTYPE_V6, LWRES_R_NOTFOUND, NULL);
 
@@ -728,16 +728,16 @@ main(void) {
 	test_gethostbyaddr("10.10.10.1", AF_INET, "ipv4.example");
 	test_gethostbyaddr("10.10.10.17", AF_INET, NULL);
 	test_gethostbyaddr("0123:4567:89ab:cdef:0123:4567:89ab:cdef",
-			   AF_INET6, "nibble.example");
+			   AF_INET6, "ip6.int.example");
 	test_gethostbyaddr("1123:4567:89ab:cdef:0123:4567:89ab:cdef",
-			   AF_INET6, "bitstring.example");
+			   AF_INET6, "ip6.arpa.example");
 
 	test_getipnodebyaddr("10.10.10.1", AF_INET, "ipv4.example");
 	test_getipnodebyaddr("10.10.10.17", AF_INET, NULL);
 	test_getipnodebyaddr("0123:4567:89ab:cdef:0123:4567:89ab:cdef",
-			     AF_INET6, "nibble.example");
+			     AF_INET6, "ip6.int.example");
 	test_getipnodebyaddr("1123:4567:89ab:cdef:0123:4567:89ab:cdef",
-			     AF_INET6, "bitstring.example");
+			     AF_INET6, "ip6.arpa.example");
 
 	test_getaddrinfo("a.example1.", AF_INET, 1, 1, "10.0.1.1");
 	test_getaddrinfo("a.example1.", AF_INET, 1, 0, "10.0.1.1");
@@ -751,27 +751,18 @@ main(void) {
 	test_getnameinfo("10.10.10.1", AF_INET, "ipv4.example");
 	test_getnameinfo("10.10.10.17", AF_INET, NULL);
 	test_getnameinfo("0123:4567:89ab:cdef:0123:4567:89ab:cdef",
-			 AF_INET6, "nibble.example");
+			 AF_INET6, "ip6.int.example");
 	test_getnameinfo("1123:4567:89ab:cdef:0123:4567:89ab:cdef",
-			 AF_INET6, "bitstring.example");
+			 AF_INET6, "ip6.arpa.example");
 	test_getnameinfo("1122:3344:5566:7788:99aa:bbcc:ddee:ff00",
 			 AF_INET6, "dname.example1");
 
-#ifdef ISC_RFC_2535
 	test_getrrsetbyname("a", 1, 1, 1, 0, 1);
 	test_getrrsetbyname("a.example1.", 1, 1, 1, 0, 1);
 	test_getrrsetbyname("e.example1.", 1, 1, 1, 1, 1);
 	test_getrrsetbyname("e.example1.", 1, 255, 1, 1, 0);
-	test_getrrsetbyname("e.example1.", 1, 24, 1, 0, 1);
+	test_getrrsetbyname("e.example1.", 1, 46, 1, 0, 1);
 	test_getrrsetbyname("", 1, 1, 0, 0, 0);
-#else
-	test_getrrsetbyname("a", 1, 1, 1, 0, 1);
-	test_getrrsetbyname("a.example1.", 1, 1, 1, 0, 1);
-	test_getrrsetbyname("e.example1.", 1, 1, 1, 0, 1);
-	test_getrrsetbyname("e.example1.", 1, 255, 1, 0, 0);
-	/* test_getrrsetbyname("e.example1.", 1, 24, 1, 0, 1); */
-	test_getrrsetbyname("", 1, 1, 0, 0, 0);
-#endif
 
 	if (fails == 0)
 		printf("I:ok\n");
