@@ -70,7 +70,7 @@
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static const char sccsid[] = "@(#)res_comp.c	8.1 (Berkeley) 6/4/93";
-static const char rcsid[] = "$Id: res_comp.c,v 1.1.2.1.4.1 2004/03/09 08:33:54 marka Exp $";
+static const char rcsid[] = "$Id: res_comp.c,v 1.3.18.2 2005/07/28 07:38:11 marka Exp $";
 #endif /* LIBC_SCCS and not lint */
 
 #include "port_before.h"
@@ -85,12 +85,13 @@ static const char rcsid[] = "$Id: res_comp.c,v 1.1.2.1.4.1 2004/03/09 08:33:54 m
 #include <unistd.h>
 #include "port_after.h"
 
-/*
+/*%
  * Expand compressed domain name 'src' to full domain name.
- * 'msg' is a pointer to the begining of the message,
- * 'eom' points to the first location after the message,
- * 'dst' is a pointer to a buffer of size 'dstsiz' for the result.
- * Return size of compressed name or -1 if there was an error.
+ *
+ * \li 'msg' is a pointer to the begining of the message,
+ * \li 'eom' points to the first location after the message,
+ * \li 'dst' is a pointer to a buffer of size 'dstsiz' for the result.
+ * \li Return size of compressed name or -1 if there was an error.
  */
 int
 dn_expand(const u_char *msg, const u_char *eom, const u_char *src,
@@ -103,10 +104,11 @@ dn_expand(const u_char *msg, const u_char *eom, const u_char *src,
 	return (n);
 }
 
-/*
+/*%
  * Pack domain name 'exp_dn' in presentation form into 'comp_dn'.
- * Return the size of the compressed name or -1.
- * 'length' is the size of the array pointed to by 'comp_dn'.
+ *
+ * \li Return the size of the compressed name or -1.
+ * \li 'length' is the size of the array pointed to by 'comp_dn'.
  */
 int
 dn_comp(const char *src, u_char *dst, int dstsiz,
@@ -117,7 +119,7 @@ dn_comp(const char *src, u_char *dst, int dstsiz,
 				 (const u_char **)lastdnptr));
 }
 
-/*
+/*%
  * Skip over a compressed domain name. Return the size or -1.
  */
 int
@@ -129,11 +131,9 @@ dn_skipname(const u_char *ptr, const u_char *eom) {
 	return (ptr - saveptr);
 }
 
-/*
+/*%
  * Verify that a domain name uses an acceptable character set.
- */
-
-/*
+ *
  * Note the conspicuous absence of ctype macros in these definitions.  On
  * non-ASCII hosts, we can't depend on string literals or ctype macros to
  * tell us anything about network-format data.  The rest of the BIND system
@@ -176,7 +176,7 @@ res_hnok(const char *dn) {
 	return (1);
 }
 
-/*
+/*%
  * hostname-like (A, MX, WKS) owners can have "*" as their first label
  * but must otherwise be as a host name.
  */
@@ -191,7 +191,7 @@ res_ownok(const char *dn) {
 	return (res_hnok(dn));
 }
 
-/*
+/*%
  * SOA RNAMEs and RP RNAMEs can have any printable character in their first
  * label, but the rest of the name has to look like a host name.
  */
@@ -219,8 +219,8 @@ res_mailok(const char *dn) {
 	return (0);
 }
 
-/*
- * This function is quite liberal, since RFC 1034's character sets are only
+/*%
+ * This function is quite liberal, since RFC1034's character sets are only
  * recommendations.
  */
 int
@@ -234,7 +234,7 @@ res_dnok(const char *dn) {
 }
 
 #ifdef BIND_4_COMPAT
-/*
+/*%
  * This module must export the following externally-visible symbols:
  *	___putlong
  *	___putshort
@@ -242,6 +242,18 @@ res_dnok(const char *dn) {
  *	__getshort
  * Note that one _ comes from C and the others come from us.
  */
+
+#ifdef SOLARIS2
+#ifdef  __putlong
+#undef  __putlong
+#endif
+#ifdef  __putshort
+#undef  __putshort
+#endif
+#pragma weak    putlong         =       __putlong
+#pragma weak    putshort        =       __putshort
+#endif /* SOLARIS2 */
+
 void __putlong(u_int32_t src, u_char *dst) { ns_put32(src, dst); }
 void __putshort(u_int16_t src, u_char *dst) { ns_put16(src, dst); }
 #ifndef __ultrix__
@@ -249,3 +261,5 @@ u_int32_t _getlong(const u_char *src) { return (ns_get32(src)); }
 u_int16_t _getshort(const u_char *src) { return (ns_get16(src)); }
 #endif /*__ultrix__*/
 #endif /*BIND_4_COMPAT*/
+
+/*! \file */
