@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2005  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1999-2002  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: ratelimiter.h,v 1.13.14.3 2004/03/08 09:04:53 marka Exp $ */
+/* $Id: ratelimiter.h,v 1.15.18.2 2005/04/29 00:17:01 marka Exp $ */
 
 #ifndef ISC_RATELIMITER_H
 #define ISC_RATELIMITER_H 1
@@ -24,8 +24,8 @@
  ***** Module Info
  *****/
 
-/*
- * A rate limiter is a mechanism for dispatching events at a limited
+/*! \file
+ * \brief A rate limiter is a mechanism for dispatching events at a limited
  * rate.  This is intended to be used when sending zone maintenance
  * SOA queries, NOTIFY messages, etc.
  */
@@ -46,13 +46,13 @@ ISC_LANG_BEGINDECLS
 isc_result_t
 isc_ratelimiter_create(isc_mem_t *mctx, isc_timermgr_t *timermgr,
 		       isc_task_t *task, isc_ratelimiter_t **ratelimiterp);
-/*
+/*%<
  * Create a rate limiter.  The execution interval is initially undefined.
  */
 
 isc_result_t
 isc_ratelimiter_setinterval(isc_ratelimiter_t *rl, isc_interval_t *interval);
-/*
+/*!<
  * Set the mininum interval between event executions.
  * The interval value is copied, so the caller need not preserve it.
  *
@@ -62,7 +62,7 @@ isc_ratelimiter_setinterval(isc_ratelimiter_t *rl, isc_interval_t *interval);
 
 void
 isc_ratelimiter_setpertic(isc_ratelimiter_t *rl, isc_uint32_t perint);
-/*
+/*%<
  * Set the number of events processed per interval timer tick.
  * If 'perint' is zero it is treated as 1.
  */
@@ -70,8 +70,10 @@ isc_ratelimiter_setpertic(isc_ratelimiter_t *rl, isc_uint32_t perint);
 isc_result_t
 isc_ratelimiter_enqueue(isc_ratelimiter_t *rl, isc_task_t *task,
 			isc_event_t **eventp);
-/*
- * Queue an event for rate-limited execution.  This is similar
+/*%<
+ * Queue an event for rate-limited execution.  
+ *
+ * This is similar
  * to doing an isc_task_send() to the 'task', except that the
  * execution may be delayed to achieve the desired rate of
  * execution.
@@ -80,50 +82,50 @@ isc_ratelimiter_enqueue(isc_ratelimiter_t *rl, isc_task_t *task,
  * must ensure that the task exists until the event is delivered.
  *
  * Requires:
- *	An interval has been set by calling
+ *\li	An interval has been set by calling
  *	isc_ratelimiter_setinterval().
  *
- *	'task' to be non NULL.
- *	'(*eventp)->ev_sender' to be NULL.
+ *\li	'task' to be non NULL.
+ *\li	'(*eventp)->ev_sender' to be NULL.
  */
 
 void
 isc_ratelimiter_shutdown(isc_ratelimiter_t *ratelimiter);
-/*
+/*%<
  * Shut down a rate limiter.
  *
  * Ensures:
- *	All events that have not yet been
+ *\li	All events that have not yet been
  * 	dispatched to the task are dispatched immediately with
- *	the ISC_EVENTATTR_CANCELED bit set in ev_attributes.
+ *	the #ISC_EVENTATTR_CANCELED bit set in ev_attributes.
  *
- *	Further attempts to enqueue events will fail with
- * 	ISC_R_SHUTTINGDOWN.
+ *\li	Further attempts to enqueue events will fail with
+ * 	#ISC_R_SHUTTINGDOWN.
  *
- *	The reatelimiter is no longer attached to its task.
+ *\li	The reatelimiter is no longer attached to its task.
  */
 
 void
 isc_ratelimiter_attach(isc_ratelimiter_t *source, isc_ratelimiter_t **target);
-/*
+/*%<
  * Attach to a rate limiter.
  */
 
 void
 isc_ratelimiter_detach(isc_ratelimiter_t **ratelimiterp);
-/*
+/*%<
  * Detach from a rate limiter.
  */
 
 isc_result_t
 isc_ratelimiter_stall(isc_ratelimiter_t *rl);
-/*
+/*%<
  * Stall event processing.
  */
 
 isc_result_t
 isc_ratelimiter_release(isc_ratelimiter_t *rl);
-/*
+/*%<
  * Release a stalled rate limiter.
  */
 
