@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2004, 2007, 2008  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 1999-2001, 2003  Internet Software Consortium.
+ * Copyright (C) 2004, 2007  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 1999-2001  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: interfaceiter.c,v 1.4.12.11 2008/09/11 06:17:28 marka Exp $ */
+/* $Id: interfaceiter.c,v 1.8.18.3 2007/08/28 07:20:06 tbox Exp $ */
 
 /*
  * Note that this code will need to be revisited to support IPv6 Interfaces.
@@ -62,13 +62,11 @@ struct isc_interfaceiter {
 	int			socket;
 	INTERFACE_INFO		IFData;		/* Current Interface Info */
 	int			numIF;		/* Current Interface count */
-	int			v4IF;		/* Number of IPv4 Interfaces */
-	INTERFACE_INFO		*buf4;		/* Buffer for WSAIoctl data. */
-	unsigned int		buf4size;	/* Bytes allocated. */
-	INTERFACE_INFO		*pos4;		/* Current offset in IF List */
-	SOCKET_ADDRESS_LIST	*buf6;
-	unsigned int		buf6size;	/* Bytes allocated. */
-	unsigned int		pos6;
+	int			totalIF;	/* Total Number
+						   of Interfaces */
+	INTERFACE_INFO		*buf;		/* Buffer for WSAIoctl data. */
+	unsigned int		bufsize;	/* Bytes allocated. */
+	INTERFACE_INFO		*pos;		/* Current offset in IF List */
 	isc_interface_t		current;	/* Current interface data. */
 	isc_result_t		result;		/* Last result code. */
 };
@@ -96,7 +94,6 @@ get_addr(unsigned int family, isc_netaddr_t *dst, struct sockaddr *src) {
 		memcpy(&dst->type.in6,
 		       &((struct sockaddr_in6 *) src)->sin6_addr,
 		       sizeof(struct in6_addr));
-		dst->zone = ((struct sockaddr_in6 *) src)->sin6_scope_id;
 		break;
 	default:
 		INSIST(0);
@@ -106,7 +103,7 @@ get_addr(unsigned int family, isc_netaddr_t *dst, struct sockaddr *src) {
 
 isc_result_t
 isc_interfaceiter_create(isc_mem_t *mctx, isc_interfaceiter_t **iterp) {
-	char strbuf[ISC_STRERRORSIZE];
+	char strbuf[ISC_STRERRORSIZE]; 
 	isc_interfaceiter_t *iter;
 	isc_result_t result;
 	int error;
@@ -123,15 +120,7 @@ isc_interfaceiter_create(isc_mem_t *mctx, isc_interfaceiter_t **iterp) {
 	InitSockets();
 
 	iter->mctx = mctx;
-	iter->buf4 = NULL;
-	iter->buf6 = NULL;
-	iter->pos4 = NULL;
-	iter->pos6 = 0;
-	iter->buf6size = 0;
-	iter->buf4size = 0;
-	iter->result = ISC_R_FAILURE;
-	iter->numIF = 0;
-	iter->v4IF = 0;
+	iter->buf = NULL;
 
 	/*
 	 * Create an unbound datagram socket to do the
@@ -139,8 +128,6 @@ isc_interfaceiter_create(isc_mem_t *mctx, isc_interfaceiter_t **iterp) {
 	 */
 	if ((iter->socket = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
 		error = WSAGetLastError();
-		if (error == WSAEAFNOSUPPORT)
-			goto inet6_only;
 		isc__strerror(error, strbuf, sizeof(strbuf));
 		UNEXPECTED_ERROR(__FILE__, __LINE__,
 				"making interface scan socket: %s",
@@ -153,17 +140,17 @@ isc_interfaceiter_create(isc_mem_t *mctx, isc_interfaceiter_t **iterp) {
 	 * Get the interface configuration, allocating more memory if
 	 * necessary.
 	 */
-	iter->buf4size = IFCONF_SIZE_INITIAL*sizeof(INTERFACE_INFO);
+	iter->bufsize = IFCONF_SIZE_INITIAL*sizeof(INTERFACE_INFO);
 
 	for (;;) {
-		iter->buf4 = isc_mem_get(mctx, iter->buf4size);
-		if (iter->buf4 == NULL) {
+		iter->buf = isc_mem_get(mctx, iter->bufsize);
+		if (iter->buf == NULL) {
 			result = ISC_R_NOMEMORY;
 			goto alloc_failure;
 		}
 
 		if (WSAIoctl(iter->socket, SIO_GET_INTERFACE_LIST,
-			     0, 0, iter->buf4, iter->buf4size,
+			     0, 0, iter->buf, iter->bufsize,
 			     &bytesReturned, 0, 0) == SOCKET_ERROR)
 		{
 			error = WSAGetLastError();
@@ -187,19 +174,19 @@ isc_interfaceiter_create(isc_mem_t *mctx, isc_interfaceiter_t **iterp) {
 			 * case and retry.
 			 */
 			if (bytesReturned > 0 &&
-			    (bytesReturned < iter->buf4size))
+			    (bytesReturned < iter->bufsize))
 				break;
 		}
-		if (iter->buf4size >= IFCONF_SIZE_MAX*sizeof(INTERFACE_INFO)) {
+		if (iter->bufsize >= IFCONF_SIZE_MAX*sizeof(INTERFACE_INFO)) {
 			UNEXPECTED_ERROR(__FILE__, __LINE__,
 					 "get interface configuration: "
 					 "maximum buffer size exceeded");
 			result = ISC_R_UNEXPECTED;
 			goto ioctl_failure;
 		}
-		isc_mem_put(mctx, iter->buf4, iter->buf4size);
+		isc_mem_put(mctx, iter->buf, iter->bufsize);
 
-		iter->buf4size += IFCONF_SIZE_INCREMENT *
+		iter->bufsize += IFCONF_SIZE_INCREMENT *
 			sizeof(INTERFACE_INFO);
 	}
 
@@ -207,92 +194,23 @@ isc_interfaceiter_create(isc_mem_t *mctx, isc_interfaceiter_t **iterp) {
 	 * A newly created iterator has an undefined position
 	 * until isc_interfaceiter_first() is called.
 	 */
-	iter->v4IF = bytesReturned/sizeof(INTERFACE_INFO);
+	iter->pos = NULL;
+	iter->result = ISC_R_FAILURE;
+	iter->numIF = 0;
+	iter->totalIF = bytesReturned/sizeof(INTERFACE_INFO);
 
-	/* We don't need the socket any more, so close it */
-	closesocket(iter->socket);
 
- inet6_only:
-	/*
-	 * Create an unbound datagram socket to do the
-	 * SIO_ADDRESS_LIST_QUERY WSAIoctl on.
-	 */
-	if ((iter->socket = socket(AF_INET6, SOCK_DGRAM, 0)) < 0) {
-		error = WSAGetLastError();
-		if (error == WSAEAFNOSUPPORT)
-			goto inet_only;
-		isc__strerror(error, strbuf, sizeof(strbuf));
-		UNEXPECTED_ERROR(__FILE__, __LINE__,
-				"making interface scan socket: %s",
-				strbuf);
-		result = ISC_R_UNEXPECTED;
-		goto ioctl_failure;
-	}
-
-	/*
-	 * Get the interface configuration, allocating more memory if
-	 * necessary.
-	 */
-	iter->buf6size = sizeof(SOCKET_ADDRESS_LIST) +
-			 IFCONF_SIZE_INITIAL*sizeof(SOCKET_ADDRESS);
-
-	for (;;) {
-		iter->buf6 = isc_mem_get(mctx, iter->buf6size);
-		if (iter->buf6 == NULL) {
-			result = ISC_R_NOMEMORY;
-			goto ioctl_failure;
-		}
-
-		if (WSAIoctl(iter->socket, SIO_ADDRESS_LIST_QUERY,
-			     0, 0, iter->buf6, iter->buf6size,
-			     &bytesReturned, 0, 0) == SOCKET_ERROR)
-		{
-			error = WSAGetLastError();
-			if (error != WSAEFAULT && error != WSAENOBUFS) {
-				errno = error;
-				isc__strerror(error, strbuf, sizeof(strbuf));
-				UNEXPECTED_ERROR(__FILE__, __LINE__,
-						 "sio address list query: %s",
-						 strbuf);
-				result = ISC_R_UNEXPECTED;
-				goto ioctl6_failure;
-			}
-			/*
-			 * EINVAL.  Retry with a bigger buffer.
-			 */
-		} else
-			break;
-
-		if (iter->buf6size >= IFCONF_SIZE_MAX*sizeof(SOCKET_ADDRESS)) {
-			UNEXPECTED_ERROR(__FILE__, __LINE__,
-					 "get interface configuration: "
-					 "maximum buffer size exceeded");
-			result = ISC_R_UNEXPECTED;
-			goto ioctl6_failure;
-		}
-		isc_mem_put(mctx, iter->buf6, iter->buf6size);
-
-		iter->buf6size += IFCONF_SIZE_INCREMENT *
-			sizeof(SOCKET_ADDRESS);
-	}
-
-	closesocket(iter->socket);
-
- inet_only:
 	iter->magic = IFITER_MAGIC;
 	*iterp = iter;
+	/* We don't need the socket any more, so close it */
+	closesocket(iter->socket);
 	return (ISC_R_SUCCESS);
 
- ioctl6_failure:
-	isc_mem_put(mctx, iter->buf6, iter->buf6size);
-
  ioctl_failure:
-	if (iter->buf4 != NULL)
-		isc_mem_put(mctx, iter->buf4, iter->buf4size);
+	isc_mem_put(mctx, iter->buf, iter->bufsize);
 
  alloc_failure:
-	if (iter->socket >= 0)
-		(void) closesocket(iter->socket);
+	(void) closesocket(iter->socket);
 
  socket_failure:
 	isc_mem_put(mctx, iter, sizeof(*iter));
@@ -308,7 +226,7 @@ isc_interfaceiter_create(isc_mem_t *mctx, isc_interfaceiter_t **iterp) {
  */
 
 static isc_result_t
-internal_current(isc_interfaceiter_t *iter) {
+internal_current(isc_interfaceiter_t *iter, int family) {
 	BOOL ifNamed = FALSE;
 	unsigned long flags;
 
@@ -316,9 +234,9 @@ internal_current(isc_interfaceiter_t *iter) {
 	REQUIRE(iter->numIF >= 0);
 
 	memset(&iter->current, 0, sizeof(iter->current));
-	iter->current.af = AF_INET;
+	iter->current.af = family;
 
-	get_addr(AF_INET, &iter->current.address,
+	get_addr(family, &iter->current.address,
 		 (struct sockaddr *)&(iter->IFData.iiAddress));
 
 	/*
@@ -348,7 +266,7 @@ internal_current(isc_interfaceiter_t *iter) {
 	 * If the interface is point-to-point, get the destination address.
 	 */
 	if ((iter->current.flags & INTERFACE_F_POINTTOPOINT) != 0) {
-		get_addr(AF_INET, &iter->current.dstaddress,
+		get_addr(family, &iter->current.dstaddress,
 		(struct sockaddr *)&(iter->IFData.iiBroadcastAddress));
 	}
 
@@ -359,43 +277,18 @@ internal_current(isc_interfaceiter_t *iter) {
 	/*
 	 * Get the network mask.
 	 */
-	get_addr(AF_INET, &iter->current.netmask,
-		 (struct sockaddr *)&(iter->IFData.iiNetmask));
+	switch (family) {
+	case AF_INET:
+		get_addr(family, &iter->current.netmask,
+			 (struct sockaddr *)&(iter->IFData.iiNetmask));
+		break;
+	case AF_INET6:
+		break;
+	}
 
 	return (ISC_R_SUCCESS);
 }
 
-static isc_result_t
-internal_current6(isc_interfaceiter_t *iter) {
-	BOOL ifNamed = FALSE;
-	int i;
-
-	REQUIRE(VALID_IFITER(iter));
-	REQUIRE(iter->pos6 >= 0);
-	REQUIRE(iter->buf6 != 0);
-
-	memset(&iter->current, 0, sizeof(iter->current));
-	iter->current.af = AF_INET6;
-
-	get_addr(AF_INET6, &iter->current.address,
-		 iter->buf6->Address[iter->pos6].lpSockaddr);
-
-	/*
-	 * Get interface flags.
-	 */
-
-	iter->current.flags = INTERFACE_F_UP;
-
-	if (ifNamed == FALSE)
-		sprintf(iter->current.name,
-			"TCP/IPv6 Interface %d", iter->pos6 + 1);
-
-	for (i = 0; i< 16; i++)
-		iter->current.netmask.type.in6.s6_addr[i] = 0xff;
-	iter->current.netmask.family = AF_INET6;
-	return (ISC_R_SUCCESS);
-}
-
 /*
  * Step the iterator to the next interface.  Unlike
  * isc_interfaceiter_next(), this may leave the iterator
@@ -405,7 +298,7 @@ internal_current6(isc_interfaceiter_t *iter) {
  */
 static isc_result_t
 internal_next(isc_interfaceiter_t *iter) {
-	if (iter->numIF >= iter->v4IF)
+	if (iter->numIF >= iter->totalIF)
 		return (ISC_R_NOMORE);
 
 	/*
@@ -414,29 +307,21 @@ internal_next(isc_interfaceiter_t *iter) {
 	 * Microsoft's implementation is peculiar for returning
 	 * the list in reverse order
 	 */
-
+	 
 	if (iter->numIF == 0)
-		iter->pos4 = (INTERFACE_INFO *)(iter->buf4 + (iter->v4IF));
+		iter->pos = (INTERFACE_INFO *)(iter->buf + (iter->totalIF));
 
-	iter->pos4--;
-	if (&(iter->pos4) < &(iter->buf4))
+	iter->pos--;
+	if (&(iter->pos) < &(iter->buf))
 		return (ISC_R_NOMORE);
 
 	memset(&(iter->IFData), 0, sizeof(INTERFACE_INFO));
-	memcpy(&(iter->IFData), iter->pos4, sizeof(INTERFACE_INFO));
+	memcpy(&(iter->IFData), iter->pos, sizeof(INTERFACE_INFO));
 	iter->numIF++;
 
 	return (ISC_R_SUCCESS);
 }
 
-static isc_result_t
-internal_next6(isc_interfaceiter_t *iter) {
-	if (iter->pos6 == 0)
-		return (ISC_R_NOMORE);
-	iter->pos6--;
-	return (ISC_R_SUCCESS);
-}
-
 isc_result_t
 isc_interfaceiter_current(isc_interfaceiter_t *iter,
 			  isc_interface_t *ifdata) {
@@ -447,13 +332,21 @@ isc_interfaceiter_current(isc_interfaceiter_t *iter,
 
 isc_result_t
 isc_interfaceiter_first(isc_interfaceiter_t *iter) {
+	isc_result_t result;
 
 	REQUIRE(VALID_IFITER(iter));
 
-	if (iter->buf6 != NULL)
-		iter->pos6 = iter->buf6->iAddressCount;
-	iter->result = ISC_R_SUCCESS;
-	return (isc_interfaceiter_next(iter));
+	iter->numIF = 0;
+	for (;;) {
+		result = internal_next(iter);
+		if (result != ISC_R_SUCCESS)
+			break;
+		result = internal_current(iter, AF_INET);
+		if (result != ISC_R_IGNORE)
+			break;
+	}
+	iter->result = result;
+	return (result);
 }
 
 isc_result_t
@@ -465,16 +358,9 @@ isc_interfaceiter_next(isc_interfaceiter_t *iter) {
 
 	for (;;) {
 		result = internal_next(iter);
-		if (result == ISC_R_NOMORE) {
-			result = internal_next6(iter);
-			if (result != ISC_R_SUCCESS)
-				break;
-			result = internal_current6(iter);
-			if (result != ISC_R_IGNORE)
-				break;
-		} else if (result != ISC_R_SUCCESS)
+		if (result != ISC_R_SUCCESS)
 			break;
-		result = internal_current(iter);
+		result = internal_current(iter,AF_INET);
 		if (result != ISC_R_IGNORE)
 			break;
 	}
@@ -489,12 +375,10 @@ isc_interfaceiter_destroy(isc_interfaceiter_t **iterp) {
 	iter = *iterp;
 	REQUIRE(VALID_IFITER(iter));
 
-	if (iter->buf4 != NULL)
-		isc_mem_put(iter->mctx, iter->buf4, iter->buf4size);
-	if (iter->buf6 != NULL)
-		isc_mem_put(iter->mctx, iter->buf6, iter->buf6size);
+	isc_mem_put(iter->mctx, iter->buf, iter->bufsize);
 
 	iter->magic = 0;
 	isc_mem_put(iter->mctx, iter, sizeof(*iter));
 	*iterp = NULL;
 }
+
